#!/usr/bin/env python

from __future__ import annotations

import argparse
import itertools
import sys
import time
import uuid
from typing import Optional, Iterator

import boto3
from botocore.config import Config

sqs_client = boto3.client("sqs", config=Config(retries={"mode": "standard"}))


def sqs_get_queue_url(queue_name: str) -> str:
    res = sqs_client.get_queue_url(QueueName=queue_name)
    return res["QueueUrl"]


def sqs_entries(f, message_group_id: Optional[str]) -> Iterator[dict]:
    while True:
        message = f.readline().strip()
        if not message:
            return None
        args = {
            "Id": str(uuid.uuid4()),
            "MessageBody": message,
        }
        if message_group_id:
            args["MessageGroupId"] = message_group_id
            args["MessageDeduplicationId"] = str(uuid.uuid4())
        yield args


def sqs_send_all(f, queue_url: str, wait: int, message_group_id: Optional[str]):
    print(f"Sending to {queue_url}")
    total_cnt = 0
    entries_it = sqs_entries(f, message_group_id)
    while True:
        entries = list(itertools.islice(entries_it, 10))
        num_messages = len(entries)
        if num_messages == 0:
            break
        for entry in entries:
            print(entry)

        sqs_client.send_message_batch(QueueUrl=queue_url, Entries=entries)
        total_cnt += num_messages
        print(f"sent {total_cnt}", file=sys.stderr)
        time.sleep(wait)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--queue-url")
    parser.add_argument("--queue-name")
    parser.add_argument("--message-group-id")
    parser.add_argument("--wait", type=int, default=1)
    args = parser.parse_args()

    queue_url = args.queue_url
    if not args.queue_url:
        if not args.queue_name:
            print(f"ERROR: queue url or queue name required.")
            exit(1)
        queue_url = sqs_get_queue_url(args.queue_name)
    sqs_send_all(sys.stdin, queue_url, args.wait, args.message_group_id)


if __name__ == "__main__":
    main()
